@@ -23,12 +23,18 @@
 /* Define if your C compiler allows void * as a function result */
 #undef HAVE_VOIDPOINTER
 
+/* Define if your C compiler allows ANSI volatile */
+#undef HAVE_VOLATILE
+
 /* Define if `union wait' is the type of the first arg to wait functions.  */
 #undef HAVE_UNION_WAIT
 
 /* Define if you have the strcasecmp function.  */
 #undef HAVE_STRCASECMP
 
+/* Define if you have the memmove function.  */
+#undef HAVE_MEMMOVE
+
 /* Define if you have GNU's getopt family of functions.  */
 #undef HAVE_GETOPTLONG
 
@@ -44,6 +50,9 @@
 /* Define if you have on_exit */
 #undef HAVE_ON_EXIT
 
+/* Define if you have setrlimit */
+#undef HAVE_SETRLIMIT
+
 /* Compute an appropriate directory for PID lock files */
 #undef PID_DIR
 
@@ -67,6 +76,9 @@
 /* Define if you want RPA support compiled in */
 #undef RPA_ENABLE
 
+/* Define if you want SDPS support compiled in */
+#undef SDPS_ENABLE
+
 /* Define if you want OPIE support compiled in */
 #undef OPIE
 
@@ -75,8 +87,12 @@
 
 /* Define if you want network security support compiled in */
 #undef NET_SECURITY
+
+/* Define if you want GSSAPI authentication */
+#undef GSSAPI
 
 /* Leave that blank line there!!  Autoheader needs it.
    If you're adding to this file, keep in mind:
    The entries are in sort -df order: alphabetical, case insensitive,
    ignoring punctuation (such as underscores).  */
+
