@@ -10,7 +10,9 @@
 #include <errno.h>
 #include <signal.h>
 #include <sys/types.h>
+#ifdef HAVE_SYS_WAIT_H
 #include <sys/wait.h>
+#endif
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h>
 #else /* !HAVE_FCNTL_H */
@@ -71,6 +73,8 @@ sigchld_handler (int sig)
     while ((pid = wait3(&status, WNOHANG, 0)) > 0)
 	continue; /* swallow 'em up. */
 #else	/* Zooks! Nothing to do but wait(), and hope we don't block... */
+    int status;
+
     wait(&status);
 #endif
 }
@@ -123,8 +127,9 @@ daemonize (const char *logfile, void (*termhook)(int))
   }
 #elif	defined(SIGTSTP)		/* BSD */
   /* change process group */
+#ifndef __EMX__
   setpgrp(0, getpid());
-
+#endif
   /* lose controlling tty */
   if ((fd = open("/dev/tty", O_RDWR)) >= 0) {
     ioctl(fd, TIOCNOTTY, (char *) 0);
@@ -132,7 +137,9 @@ daemonize (const char *logfile, void (*termhook)(int))
   }
 #else					/* SVR3 and older */
   /* change process group */
+#ifndef __EMX__
   setpgrp();
+#endif
   
   /* lose controlling tty */
   signal(SIGHUP, SIG_IGN);
