@@ -38,7 +38,14 @@
 /* clear a netBSD kernel parameter out of the way */ 
 #undef		MSGBUFSIZE
 
-#define		MSGBUFSIZE	998   	/* RFC822 limit on message line size */
+/*
+ * The RFC822 limit on message line size is just 998.  But
+ * make this *way* oversized; idiot DOS-world mailers that
+ * don't line-wrap properly often ship entire paragraphs as
+ * lines.
+ */
+#define		MSGBUFSIZE	8192
+
 #define		PASSWORDLEN	64	/* max password length */
 #define		DIGESTLEN	33	/* length of MD5 digest */
 
@@ -64,7 +71,9 @@
 /* output noise level */
 #define         O_SILENT	0	/* mute, max squelch, etc. */
 #define		O_NORMAL	1	/* user-friendly */
-#define		O_VERBOSE	2	/* excessive */
+#define		O_VERBOSE	2	/* chatty */
+#define		O_DEBUG		3	/* prolix */
+#define		O_MONITOR	O_VERBOSE
 
 #define		SIZETICKER	1024	/* print 1 dot per this many bytes */
 
@@ -82,6 +91,16 @@ typedef	char	flag;
 #define FLAG_TRUE	2
 #define FLAG_FALSE	1
 
+struct runctl
+{
+    char	*logfile;
+    char	*idfile;
+    int		poll_interval;
+    flag	use_syslog;
+    flag	invisible;
+    char	*postmaster;
+};
+
 struct idlist
 {
     char *id;
@@ -102,6 +121,30 @@ struct idlist
     struct idlist *next;
 };
 
+struct method		/* describe methods for protocol state machine */
+{
+    char *name;			/* protocol name */
+#if INET6
+    char *service;
+#else /* INET6 */
+    int	port;			/* service port */
+#endif /* INET6 */
+    flag tagged;		/* if true, generate & expect command tags */
+    flag delimited;		/* if true, accept "." message delimiter */
+    int (*parse_response)();	/* response_parsing function */
+    int (*password_canonify)();	/* canonicalize password */
+    int (*getauth)();		/* authorization fetcher */
+    int (*getrange)();		/* get message range to fetch */
+    int (*getsizes)();		/* get sizes of messages */
+    int (*is_old)();		/* check for old message */
+    int (*fetch_headers)();	/* fetch FROM headera given message */
+    int (*fetch_body)();	/* fetch a given message */
+    int (*trail)();		/* eat trailer of a message */
+    int (*delete)();		/* delete method */
+    int (*logout_cmd)();	/* logout command */
+    flag retry;			/* can getrange poll for new messages? */
+};
+
 struct hostdata		/* shared among all user connections to given server */
 {
     /* rc file data */
@@ -125,6 +168,11 @@ struct hostdata		/* shared among all user connections to given server */
     flag skip;				/* suppress poll in implicit mode? */
     flag dns;				/* do DNS lookup on multidrop? */
     flag uidl;				/* use RFC1725 UIDLs? */
+#ifdef SDPS_ENABLE
+    flag sdps;				/* use Demon Internet SDPS *ENV */
+#endif /* SDPS_ENABLE */
+    flag checkalias;                  	/* resolve aliases by comparing IPs? */
+
 
 #ifdef linux
     char *interface;
@@ -134,6 +182,7 @@ struct hostdata		/* shared among all user connections to given server */
 #endif /* linux */
 
     /* computed for internal use */
+    struct method *base_protocol;	/* relevant protocol method table */
     int poll_count;			/* count of polls so far */
     char *queryname;			/* name to attempt DNS lookup on */
     char *truename;			/* "true name" of server host */
@@ -155,7 +204,7 @@ struct query
     struct idlist *smtphunt;	/* list of SMTP hosts to try forwarding to */
     char *smtphost;		/* actual SMTP host to point to */
     char *smtpaddress;		/* address we want to force in the delivery messages */ 
-    int	antispam;		/* listener's antispam response */
+    struct idlist *antispam;	/* list of listener's antispam response */
     char *mda;			/* local MDA to pass mail to */
     char *preconnect;		/* pre-connection command to execute */
     char *postconnect;		/* post-connection command to execute */
@@ -171,18 +220,27 @@ struct query
     flag dropstatus;		/* if TRUE, drop Status lines in mail */
     flag mimedecode;		/* if TRUE, decode MIME-coded headers/coded printable*/
     int	limit;			/* limit size of retrieved messages */
+    int warnings;		/* size warning interval */
     int	fetchlimit;		/* max # msgs to get in single poll */
     int	batchlimit;		/* max # msgs to pass in single SMTP session */
     int	expunge;		/* max # msgs to pass between expunges */
+    char *properties;		/* passthrough properties for extensions */
 
     struct idlist *oldsaved, *newsaved;
 
-    /* internal use */
+    /* internal use -- per-poll state */
     flag active;		/* should we actually poll this server? */
+    char *destaddr;		/* destination host for this query */
     int errcount;		/* count transient errors in last pass */
     int smtp_socket;		/* socket descriptor for SMTP connection */
     unsigned int uid;		/* UID of user to deliver to */
+    struct idlist *skipped;	/* messages skipped on the mail server */
+
+    /* internal use -- per-message state */
+    int mimemsg;		/* bitmask indicating MIME body-type */
     char digest [DIGESTLEN];	/* md5 digest buffer */
+
+    /* internal use -- housekeeping */
     struct query *next;		/* next query control block in chain */
 };
 
@@ -198,30 +256,6 @@ struct query
 #define MULTIDROP(ctl)	(ctl->wildcard || \
 				((ctl)->localnames && (ctl)->localnames->next))
 
-struct method
-{
-    char *name;			/* protocol name */
-#if INET6
-    char *service;
-#else /* INET6 */
-    int	port;			/* service port */
-#endif /* INET6 */
-    flag tagged;		/* if true, generate & expect command tags */
-    flag delimited;		/* if true, accept "." message delimiter */
-    int (*parse_response)();	/* response_parsing function */
-    int (*password_canonify)();	/* canonicalize password */
-    int (*getauth)();		/* authorization fetcher */
-    int (*getrange)();		/* get message range to fetch */
-    int (*getsizes)();		/* get sizes of messages */
-    int (*is_old)();		/* check for old message */
-    int (*fetch_headers)();	/* fetch FROM headera given message */
-    int (*fetch_body)();	/* fetch a given message */
-    int (*trail)();		/* eat trailer of a message */
-    int (*delete)();		/* delete method */
-    int (*logout_cmd)();	/* logout command */
-    flag retry;			/* can getrange poll for new messages? */
-};
-
 /*
  * Note: tags are generated with an a%04d format from a 1-origin
  * integer sequence number.  Length 4 permits transaction numbers
@@ -243,30 +277,23 @@ extern void envquery(int, char **);
 extern int outlevel;    	/* see the O_.* constants above */
 extern int yydebug;		/* enable parse debugging */
 
-/* daemon mode control */
-extern int poll_interval;	/* poll interval in seconds */
-extern flag nodetach;		/* if TRUE, don't detach daemon process */
-extern char *logfile;		/* log file for daemon mode */
-extern flag errors_to_syslog;	/* if syslog was set */
-extern flag use_invisible;	/* if invisible was set */
-extern flag quitmode;		/* if --quit was set */
-extern flag check_only;		/* if --check was set */
-extern char *cmd_logfile;	/* if --logfile was set */
-extern int cmd_daemon;		/* if --daemon was set */
-
 /* these get computed */
 extern int batchcount;		/* count of messages sent in current batch */
 extern flag peek_capable;	/* can we read msgs without setting seen? */
 
 /* miscellaneous global controls */
+extern struct runctl run;	/* global controls for this run */
+extern flag nodetach;		/* if TRUE, don't detach daemon process */
+extern flag quitmode;		/* if --quit was set */
+extern flag check_only;		/* if --check was set */
 extern char *rcfile;		/* path name of rc file */
-extern char *idfile;		/* path name of UID file */
 extern int linelimit;		/* limit # lines retrieved per site */
 extern flag versioninfo;	/* emit only version info */
 extern char *user;		/* name of invoking user */
 extern char *home;		/* home directory of invoking user */
-extern char *fetchmailhost;	/* the name of the host running fetchmail */
 extern int pass;		/* number of re-polling pass */
+extern flag configdump;		/* dump control blocks as Python dictionary */
+extern char *fetchmailhost;	/* either "localhost" or an FQDN */
 
 /* prototypes for globally callable functions */
 
@@ -285,6 +312,7 @@ void error_at_line ();
 #endif
 
 /* driver.c: transaction support */
+void set_timeout(int);
 #if defined(HAVE_STDARG_H)
 void gen_send (int sock, const char *, ... );
 int gen_recv(int sock, char *buf, int size);
@@ -295,6 +323,25 @@ int gen_recv();
 int gen_transact ();
 #endif
 
+/* use these to track what was happening when the nonresponse timer fired */
+#define GENERAL_WAIT	0	/* unknown wait type */
+#define OPEN_WAIT	1	/* waiting from mailserver open */
+#define SERVER_WAIT	2	/* waiting for mailserver response */
+#define LISTENER_WAIT	3	/* waiting for listener initialization */
+#define FORWARDING_WAIT	4	/* waiting for listener response */
+extern int phase;
+
+/* mark values for name lists */
+#define XMIT_ACCEPT		1
+#define XMIT_REJECT		2
+#define XMIT_ANTISPAM		3
+
+/* sink.c: forwarding */
+int stuffline(struct query *, char *);
+int open_sink(struct query*, char*, struct idlist*, long reallen, int*, int*);
+void release_sink(struct query *);
+int close_sink(struct query *, flag);
+
 /* rfc822.c: RFC822 header parsing */
 char *reply_hack(char *, const char *);
 char *nxtaddr(const char *);
@@ -306,7 +353,7 @@ void free_str_list(struct idlist **);
 void save_str_pair(struct idlist **, const char *, const char *);
 void free_str_pair_list(struct idlist **);
 int delete_str(struct idlist **, int);
-int str_in_list(struct idlist **, const char *);
+int str_in_list(struct idlist **, const char *, const flag);
 int str_nr_in_list(struct idlist **, const char *);
 int str_nr_last_in_list(struct idlist **, const char *);
 void str_set_mark( struct idlist **, const char *, const flag);
@@ -321,7 +368,7 @@ void write_saved_lists(struct query *, const char *);
 
 /* rcfile_y.y */
 int prc_parse_file(const char *, const flag);
-int prc_filecheck(const char *);
+int prc_filecheck(const char *, const flag);
 
 /* base64.c */
 void to64frombits(unsigned char *, const unsigned char *, int);
@@ -333,7 +380,7 @@ int from64tobits(char *, const char *);
 #define MSG_IS_8BIT       0x02
 #define MSG_NEEDS_DECODE  0x80
 extern void UnMimeHeader(unsigned char *buf);
-extern int  MimeBodyType(unsigned char *hdrs);
+extern int  MimeBodyType(unsigned char *hdrs, int WantDecode);
 extern int  UnMimeBodyline(unsigned char **buf, int collapsedoubledot);
 
 /* interface.c */
@@ -350,6 +397,14 @@ int interface_approve(struct hostdata *);
 XMALLOCTYPE *xmalloc(int);
 XMALLOCTYPE *xrealloc(XMALLOCTYPE *, int);
 char *xstrdup(const char *);
+#if defined(HAVE_ALLOCA_H)
+#include <alloca.h>
+#else
+#ifdef _AIX
+ #pragma alloca
+#endif
+#endif
+#define	xalloca(ptr, t, n)	if (!(ptr = (t) alloca(n))) error(PS_UNDEFINED, 0, "alloca failed")
 
 /* protocol driver and methods */
 int do_protocol(struct query *, const struct method *);
@@ -360,8 +415,7 @@ int doETRN (struct query *);
 
 /* miscellanea */
 struct query *hostalloc(struct query *); 
-int parsecmdline (int, char **, struct query *);
-void optmerge(struct query *, struct query *);
+int parsecmdline (int, char **, struct runctl *, struct query *);
 char *MD5Digest (unsigned char *);
 int POP3_auth_rpa(unsigned char *, unsigned char *, int socket);
 int daemonize(const char *, void (*)(int));
@@ -369,6 +423,12 @@ char *getpassword(char *);
 void escapes(const char *, char *);
 char *visbuf(const char *);
 char *showproto(int);
+void dump_config(struct runctl *runp, struct query *querylist);
+int is_host_alias(const char *, struct query *);
+char *host_fqdn(void);
+#ifdef SDPS_ENABLE
+char *sdps_envto;
+#endif /* SDPS_ENABLE */
 
 void yyerror(const char *);
 int yylex(void);
@@ -387,5 +447,6 @@ void itimerthread(void*);
 #endif
 
 #define STRING_DISABLED	(char *)-1
+#define STRING_DUMMY	""
 
 /* fetchmail.h ends here */
