@@ -20,7 +20,7 @@
 #include <des.h>
 #define krb_get_err_text(e) (krb_err_txt[e])
 #endif
-#if defined (__FreeBSD__) || defined(__linux__)
+#if defined(__NetBSD__) || (__FreeBSD__) || defined(__linux__)
 #define krb_get_err_text(e) (krb_err_txt[e])
 #endif
 #include <krb.h>
@@ -123,63 +123,63 @@ int imap_ok(int sock, char *argbuf)
 #if OPIE
 static int do_otp(int sock, struct query *ctl)
 {
-  int i, rval;
-  char buffer[128];
-  char challenge[OPIE_CHALLENGE_MAX+1];
-  char response[OPIE_RESPONSE_MAX+1];
+    int i, rval;
+    char buffer[128];
+    char challenge[OPIE_CHALLENGE_MAX+1];
+    char response[OPIE_RESPONSE_MAX+1];
 
-  gen_send(sock, "AUTHENTICATE X-OTP");
+    gen_send(sock, "AUTHENTICATE X-OTP");
 
-  if (rval = gen_recv(sock, buffer, sizeof(buffer)))
-    return rval;
+    if (rval = gen_recv(sock, buffer, sizeof(buffer)))
+	return rval;
 
-  if ((i = from64tobits(challenge, buffer)) < 0) {
-    error(0, -1, "Could not decode initial BASE64 challenge");
-    return PS_AUTHFAIL;
-  };
+    if ((i = from64tobits(challenge, buffer)) < 0) {
+	error(0, -1, "Could not decode initial BASE64 challenge");
+	return PS_AUTHFAIL;
+    };
 
 
-  to64frombits(buffer, ctl->remotename, strlen(ctl->remotename));
+    to64frombits(buffer, ctl->remotename, strlen(ctl->remotename));
 
-  if (outlevel == O_VERBOSE)
-     error(0, 0, "IMAP> %s", buffer);
-  SockWrite(sock, buffer, strlen(buffer));
-  SockWrite(sock, "\r\n", 2);
+    if (outlevel >= O_MONITOR)
+	error(0, 0, "IMAP> %s", buffer);
+    SockWrite(sock, buffer, strlen(buffer));
+    SockWrite(sock, "\r\n", 2);
 
-  if (rval = gen_recv(sock, buffer, sizeof(buffer)))
-    return rval;
+    if (rval = gen_recv(sock, buffer, sizeof(buffer)))
+	return rval;
 
-  if ((i = from64tobits(challenge, buffer)) < 0) {
-    error(0, -1, "Could not decode OTP challenge");
-    return PS_AUTHFAIL;
-  };
+    if ((i = from64tobits(challenge, buffer)) < 0) {
+	error(0, -1, "Could not decode OTP challenge");
+	return PS_AUTHFAIL;
+    };
 
-  rval = opiegenerator(challenge, !strcmp(ctl->password, "opie") ? "" : ctl->password, response);
-  if ((rval == -2) && (cmd_daemon == -1)) {
-    char secret[OPIE_SECRET_MAX+1];
-    fprintf(stderr, "Secret pass phrase: ");
-    if (opiereadpass(secret, sizeof(secret), 0))
-      rval = opiegenerator(challenge, secret, response);
-    memset(secret, 0, sizeof(secret));
-  };
+    rval = opiegenerator(challenge, !strcmp(ctl->password, "opie") ? "" : ctl->password, response);
+    if ((rval == -2) && !run.poll_interval) {
+	char secret[OPIE_SECRET_MAX+1];
+	fprintf(stderr, "Secret pass phrase: ");
+	if (opiereadpass(secret, sizeof(secret), 0))
+	    rval = opiegenerator(challenge, secret, response);
+	memset(secret, 0, sizeof(secret));
+    };
 
-  if (rval)
-   return PS_AUTHFAIL;
+    if (rval)
+	return PS_AUTHFAIL;
 
-  to64frombits(buffer, response, strlen(response));
+    to64frombits(buffer, response, strlen(response));
 
-  if (outlevel == O_VERBOSE)
-     error(0, 0, "IMAP> %s", buffer);
-  SockWrite(sock, buffer, strlen(buffer));
-  SockWrite(sock, "\r\n", 2);
+    if (outlevel >= O_MONITOR)
+	error(0, 0, "IMAP> %s", buffer);
+    SockWrite(sock, buffer, strlen(buffer));
+    SockWrite(sock, "\r\n", 2);
 
-  if (rval = gen_recv(sock, buffer, sizeof(buffer)))
-    return rval;
+    if (rval = gen_recv(sock, buffer, sizeof(buffer)))
+	return rval;
 
-  if (strstr(buffer, "OK"))
-    return PS_SUCCESS;
-  else
-    return PS_AUTHFAIL;
+    if (strstr(buffer, "OK"))
+	return PS_SUCCESS;
+    else
+	return PS_AUTHFAIL;
 };
 #endif /* OPIE */
 
@@ -308,7 +308,7 @@ static int do_rfc1731(int sock, char *truename)
     }
 
     to64frombits(buf1, authenticator.dat, authenticator.length);
-    if (outlevel == O_VERBOSE) {
+    if (outlevel >= O_MONITOR) {
 	error(0, 0, "IMAP> %s", buf1);
     }
     SockWrite(sock, buf1, strlen(buf1));
@@ -391,7 +391,7 @@ static int do_rfc1731(int sock, char *truename)
 	    &session, 1);
 
     to64frombits(buf1, authenticator.dat, authenticator.length);
-    if (outlevel == O_VERBOSE) {
+    if (outlevel >= O_MONITOR) {
 	error(0, 0, "IMAP> %s", buf1);
     }
     SockWrite(sock, buf1, strlen(buf1));
@@ -438,7 +438,7 @@ static int do_gssauth(int sock, char *hostname, char *username)
         error(0, -1, "Couldn't get service name for [%s]", buf1);
         return PS_AUTHFAIL;
     }
-    else if (outlevel == O_VERBOSE) {
+    else if (outlevel >= O_DEBUG) {
         maj_stat = gss_display_name(&min_stat, target_name, &request_buf,
             &mech_name);
         error(0, 0, "Using service name [%s]",request_buf.value);
@@ -456,7 +456,7 @@ static int do_gssauth(int sock, char *hostname, char *username)
     /* now start the security context initialisation loop... */
     sec_token = GSS_C_NO_BUFFER;
     context = GSS_C_NO_CONTEXT;
-    if (outlevel == O_VERBOSE)
+    if (outlevel >= O_VERBOSE)
         error(0,0,"Sending credentials");
     do {
         maj_stat = gss_init_sec_context(&min_stat, GSS_C_NO_CREDENTIAL, 
@@ -475,7 +475,7 @@ static int do_gssauth(int sock, char *hostname, char *username)
         gss_release_buffer(&min_stat, &send_token);
         SockWrite(sock, buf1, strlen(buf1));
         SockWrite(sock, "\r\n", 2);
-        if (outlevel == O_VERBOSE)
+        if (outlevel >= O_MONITOR)
             error(0,0,"IMAP> %s", buf1);
         if (maj_stat == GSS_S_CONTINUE_NEEDED) {
 	    if (result = gen_recv(sock, buf1, sizeof buf1)) {
@@ -503,7 +503,7 @@ static int do_gssauth(int sock, char *hostname, char *username)
         gss_release_buffer(&min_stat, &send_token);
         return PS_AUTHFAIL;
     }
-    if (outlevel == O_VERBOSE)
+    if (outlevel >= O_DEBUG)
         error(0,0,"Credential exchange complete");
     /* first octet is security levels supported. We want none, for now */
     server_conf_flags = ((char *)send_token.value)[0];
@@ -516,7 +516,7 @@ static int do_gssauth(int sock, char *hostname, char *username)
     buf_size = ntohl(*((long *)send_token.value));
     /* we don't care about buffer size if we don't wrap data */
     gss_release_buffer(&min_stat, &send_token);
-    if (outlevel == O_VERBOSE) {
+    if (outlevel >= O_DEBUG) {
         error(0,0,"Unwrapped security level flags: %s%s%s",
             server_conf_flags & GSSAUTH_P_NONE ? "N" : "-",
             server_conf_flags & GSSAUTH_P_INTEGRITY ? "I" : "-",
@@ -538,7 +538,7 @@ static int do_gssauth(int sock, char *hostname, char *username)
         return PS_AUTHFAIL;
     }
     to64frombits(buf1, send_token.value, send_token.length);
-    if (outlevel == O_VERBOSE) {
+    if (outlevel >= O_DEBUG) {
         error(0,0,"Requesting authorisation as %s", username);
         error(0,0,"IMAP> %s",buf1);
     }
@@ -550,7 +550,7 @@ static int do_gssauth(int sock, char *hostname, char *username)
         return result;
     if (strstr(buf1, "OK")) {
         /* flush security context */
-        if (outlevel == O_VERBOSE)
+        if (outlevel >= O_DEBUG)
             error(0, 0, "Releasing GSS credentials");
         maj_stat = gss_delete_sec_context(&min_stat, &context, &send_token);
         if (maj_stat != GSS_S_COMPLETE) {
@@ -561,6 +561,7 @@ static int do_gssauth(int sock, char *hostname, char *username)
          * credentials. RFC 1731 doesn't specify what to do, and since this
          * support is only for authentication, we'll assume the server
          * knows enough to flush its own credentials */
+        gss_release_buffer(&min_stat, &send_token);
         return PS_SUCCESS;
     }
 
@@ -599,20 +600,20 @@ int imap_getauth(int sock, struct query *ctl, char *greeting)
 	if (strstr(capabilities, "IMAP4REV1"))
 	{
 	    imap_version = IMAP4rev1;
-	    if (outlevel == O_VERBOSE)
+	    if (outlevel >= O_DEBUG)
 		error(0, 0, "Protocol identified as IMAP4 rev 1");
 	}
 	else
 	{
 	    imap_version = IMAP4;
-	    if (outlevel == O_VERBOSE)
-	    error(0, 0, "Protocol identified as IMAP4 rev 0");
+	    if (outlevel >= O_DEBUG)
+		error(0, 0, "Protocol identified as IMAP4 rev 0");
 	}
     }
     else if (ok == PS_ERROR)
     {
 	imap_version = IMAP2;
-	if (outlevel == O_VERBOSE)
+	if (outlevel >= O_DEBUG)
 	    error(0, 0, "Protocol identified as IMAP2 or IMAP2BIS");
     }
     else
@@ -621,11 +622,12 @@ int imap_getauth(int sock, struct query *ctl, char *greeting)
     peek_capable = (imap_version >= IMAP4);
 
 #if OPIE
-    if ((ctl->server.protocol == P_IMAP) && strstr(capabilities, "AUTH=X-OTP")) {
-      if (outlevel == O_VERBOSE)
-        error(0, 0, "OTP authentication is supported");
-      if (do_otp(sock, ctl) == PS_SUCCESS)
-        return PS_SUCCESS;
+    if ((ctl->server.protocol == P_IMAP) && strstr(capabilities, "AUTH=X-OTP"))
+    {
+	if (outlevel >= O_DEBUG)
+	    error(0, 0, "OTP authentication is supported");
+	if (do_otp(sock, ctl) == PS_SUCCESS)
+	    return(PS_SUCCESS);
     };
 #endif /* OPIE */
 
@@ -634,7 +636,7 @@ int imap_getauth(int sock, struct query *ctl, char *greeting)
     {
         if (ctl->server.protocol == P_IMAP_GSS)
         {
-            if (outlevel == O_VERBOSE)
+            if (outlevel >= O_DEBUG)
                 error(0, 0, "GSS authentication is supported");
             return do_gssauth(sock, ctl->server.truename, ctl->remotename);
         }
@@ -649,14 +651,14 @@ int imap_getauth(int sock, struct query *ctl, char *greeting)
 #ifdef KERBEROS_V4
     if (strstr(capabilities, "AUTH=KERBEROS_V4"))
     {
-	if (outlevel == O_VERBOSE)
+	if (outlevel >= O_DEBUG)
 	    error(0, 0, "KERBEROS_V4 authentication is supported");
 
 	if (ctl->server.protocol == P_IMAP_K4)
 	{
 	    if ((ok = do_rfc1731(sock, ctl->server.truename)))
 	    {
-		if (outlevel == O_VERBOSE)
+		if (outlevel >= O_MONITOR)
 		    error(0, 0, "IMAP> *");
 		SockWrite(sock, "*\r\n", 3);
 	    }
