@@ -9,8 +9,12 @@
 #include <stdio.h>
 #include <pwd.h>
 #include <string.h>
+#include <errno.h>
 #if defined(STDC_HEADERS)
 #include  <stdlib.h>
+#include  <limits.h>
+#else
+#include  <ctype.h>
 #endif
 
 #include "getopt.h"
@@ -27,38 +31,42 @@
 #define LA_LOGFILE	9
 #define LA_INVISIBLE	10
 #define LA_SYSLOG	11
-#define LA_RCFILE	12
-#define LA_IDFILE	13
-#define LA_PROTOCOL	14
-#define LA_UIDL		15
-#define LA_PORT		16
-#define LA_AUTHENTICATE	17
-#define LA_TIMEOUT	18
-#define LA_ENVELOPE	19
-#define LA_QVIRTUAL     20
-#define LA_USERNAME	21
-#define LA_ALL          22
-#define LA_NOKEEP	23
-#define	LA_KEEP		24
-#define LA_FLUSH        25
-#define LA_NOREWRITE	26
-#define LA_LIMIT	27
-#define LA_FOLDER	28
-#define LA_SMTPHOST	29
-#define LA_SMTPADDR     30
-#define LA_ANTISPAM	31
-#define LA_BATCHLIMIT	32
-#define LA_FETCHLIMIT	33
-#define LA_EXPUNGE	34
-#define LA_MDA		35
-#define LA_NETSEC	36
-#define LA_INTERFACE    37
-#define LA_MONITOR      38
-#define LA_YYDEBUG	39
+#define LA_NOSYSLOG	12
+#define LA_RCFILE	13
+#define LA_IDFILE	14
+#define LA_POSTMASTER	15
+#define LA_PROTOCOL	16
+#define LA_UIDL		17
+#define LA_PORT		18
+#define LA_AUTHENTICATE	19
+#define LA_TIMEOUT	20
+#define LA_ENVELOPE	21
+#define LA_QVIRTUAL     22
+#define LA_USERNAME	23
+#define LA_ALL          24
+#define LA_NOKEEP	25
+#define	LA_KEEP		26
+#define LA_FLUSH        27
+#define LA_NOREWRITE	28
+#define LA_LIMIT	29
+#define LA_WARNINGS	30
+#define LA_FOLDER	31
+#define LA_SMTPHOST	32
+#define LA_SMTPADDR     33
+#define LA_ANTISPAM	34
+#define LA_BATCHLIMIT	35
+#define LA_FETCHLIMIT	36
+#define LA_EXPUNGE	37
+#define LA_MDA		38
+#define LA_NETSEC	39
+#define LA_INTERFACE    40
+#define LA_MONITOR      41
+#define LA_CONFIGDUMP	42
+#define LA_YYDEBUG	43
 
-/* options still left: CgGhHjJoORUwWxXYz */
+/* options still left: CgGhHjJoORwWxXYz */
 static const char *shortoptions = 
-	"?Vcsvd:NqL:f:i:p:UP:A:t:E:Q:u:akKFnl:r:S:Z:b:B:e:m:T:I:M:y";
+	"?Vcsvd:NqL:f:i:p:UP:A:t:E:Q:u:akKFnl:r:S:Z:b:B:e:m:T:I:M:yw:";
 
 static const struct option longoptions[] = {
 /* this can be const because all flag fields are 0 and will never get set */
@@ -73,8 +81,10 @@ static const struct option longoptions[] = {
   {"logfile",	required_argument, (int *) 0, LA_LOGFILE     },
   {"invisible",	no_argument,	   (int *) 0, LA_INVISIBLE   },
   {"syslog",	no_argument,	   (int *) 0, LA_SYSLOG      },
+  {"nosyslog",	no_argument,	   (int *) 0, LA_NOSYSLOG    },
   {"fetchmailrc",required_argument,(int *) 0, LA_RCFILE      },
   {"idfile",	required_argument, (int *) 0, LA_IDFILE      },
+  {"postmaster",required_argument, (int *) 0, LA_POSTMASTER  },
 
   {"protocol",	required_argument, (int *) 0, LA_PROTOCOL    },
   {"proto",	required_argument, (int *) 0, LA_PROTOCOL    },
@@ -94,6 +104,7 @@ static const struct option longoptions[] = {
   {"flush",	no_argument,	   (int *) 0, LA_FLUSH       },
   {"norewrite",	no_argument,	   (int *) 0, LA_NOREWRITE   },
   {"limit",	required_argument, (int *) 0, LA_LIMIT       },
+  {"warnings",	required_argument, (int *) 0, LA_WARNINGS    },
 
   {"folder",    required_argument, (int *) 0, LA_FOLDER	     },
   {"smtphost",	required_argument, (int *) 0, LA_SMTPHOST    },
@@ -114,15 +125,89 @@ static const struct option longoptions[] = {
   {"monitor",	required_argument, (int *) 0, LA_MONITOR     },
 #endif /* defined(linux) && !INET6 */
 
+  {"configdump",no_argument,	   (int *) 0, LA_CONFIGDUMP  },
+
   {"yydebug",	no_argument,	   (int *) 0, LA_YYDEBUG     },
 
   {(char *) 0,  no_argument,       (int *) 0, 0              }
 };
 
-int parsecmdline (argc, argv, ctl)
+static int xatoi(char *s, int *errflagptr)
+/* do safe conversion from string to number */
+{
+#if defined (STDC_HEADERS) && defined (LONG_MAX) && defined (INT_MAX)
+    /* parse and convert numbers, but also check for invalid characters in
+     * numbers
+     */
+
+    char *endptr;
+    long value;
+
+    errno = 0;
+
+    value = strtol(s, &endptr, 0);
+
+    /* any invalid chars in string? */
+    if ( (endptr == s) || (*endptr != '\0') ) {
+    	(void) fprintf(stderr, "String '%s' is not a valid number string.\n", s);
+	(*errflagptr)++;
+	return 0;
+    }
+
+    /* is the range valid? */
+    if ( (((value == LONG_MAX) || (value == LONG_MIN)) && (errno == ERANGE)) ||
+				(value > INT_MAX) || (value < INT_MIN)) {
+
+    	(void) fprintf(stderr, "Value of string '%s' is %s than %d.\n", s,
+					(value < 0) ? "smaller": "larger",
+					(value < 0) ? INT_MIN : INT_MAX);
+	(*errflagptr)++;
+	return 0;
+    }
+
+    return (int) value;  /* shut up, I know what I'm doing */
+#else
+    int	i;
+    char *dp;
+# if defined (STDC_HEADERS)
+    size_t	len;
+# else
+    int		len;
+# endif
+
+    /* We do only base 10 conversions here (atoi)! */
+
+    len = strlen(s);
+    /* check for leading white spaces */
+    for (i = 0; (i < len) && isspace(s[i]); i++)
+    	;
+
+    dp = &s[i];
+
+    /* check for +/- */
+    if (i < len && (s[i] == '+' || s[i] == '-'))	i++;
+
+    /* skip over digits */
+    for ( /* no init */ ; (i < len) && isdigit(s[i]); i++)
+    	;
+
+    /* check for trailing garbage */
+    if (i != len) {
+    	(void) fprintf(stderr, "String '%s' is not a valid number string.\n", s);
+    	(*errflagptr)++;
+	return 0;
+    }
+
+    /* atoi should be safe by now, except for number range over/underflow */
+    return atoi(dp);
+#endif
+}
+
+int parsecmdline (argc, argv, rctl, ctl)
 /* parse and validate the command line options */
 int argc;		/* argument count */
 char **argv;		/* argument strings */
+struct runctl *rctl;	/* global run controls to modify */
 struct query *ctl;	/* option record to be initialized */
 {
     /*
@@ -137,9 +222,9 @@ struct query *ctl;	/* option record to be initialized */
     int ocount = 0;     /* count of destinations specified */
     int errflag = 0;   /* TRUE when a syntax error is detected */
     int option_index;
-    char buf[BUFSIZ], *cp;
+    char *buf, *cp;
 
-    cmd_daemon = -1;
+    rctl->poll_interval = -1;
 
     memset(ctl, '\0', sizeof(struct query));    /* start clean */
     ctl->smtp_socket = -1;
@@ -163,11 +248,14 @@ struct query *ctl;	/* option record to be initialized */
 	    break;
 	case 'v':
 	case LA_VERBOSE:
-	    outlevel = O_VERBOSE;
+	    if (outlevel == O_VERBOSE)
+		outlevel = O_DEBUG;
+	    else
+		outlevel = O_VERBOSE;
 	    break;
 	case 'd':
 	case LA_DAEMON:
-	    cmd_daemon = atoi(optarg);
+	    rctl->poll_interval = xatoi(optarg, &errflag);
 	    break;
 	case 'N':
 	case LA_NODETACH:
@@ -179,10 +267,10 @@ struct query *ctl;	/* option record to be initialized */
 	    break;
 	case 'L':
 	case LA_LOGFILE:
-	    cmd_logfile = optarg;
+	    rctl->logfile = optarg;
 	    break;
 	case LA_INVISIBLE:
-	    use_invisible = TRUE;
+	    rctl->invisible = TRUE;
 	    break;
 	case 'f':
 	case LA_RCFILE:
@@ -191,14 +279,24 @@ struct query *ctl;	/* option record to be initialized */
 	    break;
 	case 'i':
 	case LA_IDFILE:
-	    idfile = (char *) xmalloc(strlen(optarg)+1);
-	    strcpy(idfile,optarg);
+	    rctl->idfile = (char *) xmalloc(strlen(optarg)+1);
+	    strcpy(rctl->idfile,optarg);
+	    break;
+	case LA_POSTMASTER:
+	    rctl->postmaster = (char *) xmalloc(strlen(optarg)+1);
 	    break;
 	case 'p':
 	case LA_PROTOCOL:
 	    /* XXX -- should probably use a table lookup here */
 	    if (strcasecmp(optarg,"pop2") == 0)
 		ctl->server.protocol = P_POP2;
+#ifdef SDPS_ENABLE
+	    else if (strcasecmp(optarg,"sdps") == 0)
+	    {
+	        ctl->server.protocol = P_POP3; 
+                ctl->server.sdps = TRUE;
+	    }
+#endif /* SDPS_ENABLE */
 	    else if (strcasecmp(optarg,"pop3") == 0)
 		ctl->server.protocol = P_POP3;
 	    else if (strcasecmp(optarg,"apop") == 0)
@@ -225,6 +323,10 @@ struct query *ctl;	/* option record to be initialized */
 	    else if (strcasecmp(optarg,"imap-k4") == 0)
 		ctl->server.protocol = P_IMAP_K4;
 #endif /* KERBEROS_V4 */
+#ifdef GSSAPI
+	    else if (strcasecmp(optarg, "imap-gss") == 0)
+                ctl->server.protocol = P_IMAP_GSS;
+#endif /* GSSAPI */
 	    else if (strcasecmp(optarg,"etrn") == 0)
 		ctl->server.protocol = P_ETRN;
 	    else {
@@ -241,7 +343,7 @@ struct query *ctl;	/* option record to be initialized */
 #if INET6
 	    ctl->server.service = optarg;
 #else /* INET6 */
-	    ctl->server.port = atoi(optarg);
+	    ctl->server.port = xatoi(optarg, &errflag);
 #endif /* INET6 */
 	    break;
 	case 'A':
@@ -265,7 +367,7 @@ struct query *ctl;	/* option record to be initialized */
 	    break;
 	case 't':
 	case LA_TIMEOUT:
-	    ctl->server.timeout = atoi(optarg);
+	    ctl->server.timeout = xatoi(optarg, &errflag);
 	    if (ctl->server.timeout == 0)
 		ctl->server.timeout = -1;
 	    break;
@@ -304,11 +406,12 @@ struct query *ctl;	/* option record to be initialized */
 	    break;
 	case 'l':
 	case LA_LIMIT:
-	    c = atoi(optarg);
+	    c = xatoi(optarg, &errflag);
 	    ctl->limit = NUM_VALUE(c);
 	    break;
 	case 'r':
 	case LA_FOLDER:
+	    xalloca(buf, char *, strlen(optarg));
 	    strcpy(buf, optarg);
 	    cp = strtok(buf, ",");
 	    do {
@@ -318,6 +421,7 @@ struct query *ctl;	/* option record to be initialized */
 	    break;
 	case 'S':
 	case LA_SMTPHOST:
+	    xalloca(buf, char *, strlen(optarg));
 	    strcpy(buf, optarg);
 	    cp = strtok(buf, ",");
 	    do {
@@ -332,21 +436,29 @@ struct query *ctl;	/* option record to be initialized */
 	    break;
 	case 'Z':
 	case LA_ANTISPAM:
-	    c = atoi(optarg);
-	    ctl->antispam = NUM_VALUE(c);
+	    xalloca(buf, char *, strlen(optarg));
+	    strcpy(buf, optarg);
+	    cp = strtok(buf, ",");
+	    do {
+		struct idlist	*idp = save_str(&ctl->antispam, NULL, 0);;
+
+		idp->val.status.num = atoi(cp);
+	    } while
+		((cp = strtok((char *)NULL, ",")));
+	    break;
 	case 'b':
 	case LA_BATCHLIMIT:
-	    c = atoi(optarg);
+	    c = xatoi(optarg, &errflag);
 	    ctl->batchlimit = NUM_VALUE(c);
 	    break;
 	case 'B':
 	case LA_FETCHLIMIT:
-	    c = atoi(optarg);
+	    c = xatoi(optarg, &errflag);
 	    ctl->fetchlimit = NUM_VALUE(c);
 	    break;
 	case 'e':
 	case LA_EXPUNGE:
-	    c = atoi(optarg);
+	    c = xatoi(optarg, &errflag);
 	    ctl->expunge = NUM_VALUE(c);
 	    break;
 	case 'm':
@@ -381,8 +493,22 @@ struct query *ctl;	/* option record to be initialized */
 	    yydebug = TRUE;
 	    break;
 
+	case 'w':
+	case LA_WARNINGS:
+	    c = xatoi(optarg, &errflag);
+	    ctl->warnings = NUM_VALUE(c);
+	    break;
+
+	case LA_CONFIGDUMP:
+	    configdump = TRUE;
+	    break;
+
 	case LA_SYSLOG:
-	    errors_to_syslog = TRUE;
+	    rctl->use_syslog = FLAG_TRUE;
+	    break;
+
+	case LA_NOSYSLOG:
+	    rctl->use_syslog = FLAG_FALSE;
 	    break;
 
 	case '?':
@@ -394,57 +520,57 @@ struct query *ctl;	/* option record to be initialized */
 
     if (errflag || ocount > 1) {
 	/* squawk if syntax errors were detected */
-	fputs("usage:  fetchmail [options] [server ...]\n", stderr);
-	fputs("  Options are as follows:\n",stderr);
-	fputs("  -?, --help        display this option help\n", stderr);
-	fputs("  -V, --version     display version info\n", stderr);
+#define P(s)	fputs(s, stderr)
+	P("usage:  fetchmail [options] [server ...]\n");
+	P("  Options are as follows:\n");
+	P("  -?, --help        display this option help\n");
+	P("  -V, --version     display version info\n");
 
-	fputs("  -c, --check       check for messages without fetching\n", stderr);
-	fputs("  -s, --silent      work silently\n", stderr);
-	fputs("  -v, --verbose     work noisily (diagnostic output)\n", stderr);
-	fputs("  -d, --daemon      run as a daemon once per n seconds\n", stderr);
-	fputs("  -N, --nodetach    don't detach daemon process\n", stderr);
-	fputs("  -q, --quit        kill daemon process\n", stderr);
-	fputs("  -L, --logfile     specify logfile name\n", stderr);
-	fputs("      --syslog      use syslog(3) for most messages when running as a daemon\n", stderr);
-	fputs("      --invisible   suppress Received line & enable host spoofing\n", stderr);
-	fputs("  -f, --fetchmailrc specify alternate run control file\n", stderr);
-	fputs("  -i, --idfile      specify alternate UIDs file\n", stderr);
+	P("  -c, --check       check for messages without fetching\n");
+	P("  -s, --silent      work silently\n");
+	P("  -v, --verbose     work noisily (diagnostic output)\n");
+	P("  -d, --daemon      run as a daemon once per n seconds\n");
+	P("  -N, --nodetach    don't detach daemon process\n");
+	P("  -q, --quit        kill daemon process\n");
+	P("  -L, --logfile     specify logfile name\n");
+	P("      --syslog      use syslog(3) for most messages when running as a daemon\n");
+	P("      --invisible   don't write Received & enable host spoofing\n");
+	P("  -f, --fetchmailrc specify alternate run control file\n");
+	P("  -i, --idfile      specify alternate UIDs file\n");
+	P("      --postmaster  specify recipient of last resort\n");
 #if defined(linux) && !INET6
-	fputs("  -I, --interface   interface required specification\n",stderr);
-	fputs("  -M, --monitor     monitor interface for activity\n",stderr);
+	P("  -I, --interface   interface required specification\n");
+	P("  -M, --monitor     monitor interface for activity\n");
 #endif
 
-#ifdef KERBEROS_V4
-	fputs("  -p, --protocol    specify pop2, pop3, imap, apop, rpop, kpop, etrn, imap-k4\n", stderr);
-#else
-	fputs("  -p, --protocol    specify pop2, pop3, imap, apop, rpop, kpop, etrn\n", stderr);
-#endif /* KERBEROS_V4 */
-	fputs("  -U, --uidl        force the use of UIDLs (pop3 only)\n", stderr);
-	fputs("  -P, --port        TCP/IP service port to connect to\n",stderr);
-	fputs("  -A, --auth        authentication type (password or kerberos)\n",stderr);
-	fputs("  -t, --timeout     server nonresponse timeout\n",stderr);
-	fputs("  -E, --envelope    envelope address header\n",stderr);
-	fputs("  -Q, --qvirtual    prefix to remove from local user id\n",stderr);
+	P("  -p, --protocol    specify retrieval protocol (see man page)\n");
+	P("  -U, --uidl        force the use of UIDLs (pop3 only)\n");
+	P("  -P, --port        TCP/IP service port to connect to\n");
+	P("  -A, --auth        authentication type (password or kerberos)\n");
+	P("  -t, --timeout     server nonresponse timeout\n");
+	P("  -E, --envelope    envelope address header\n");
+	P("  -Q, --qvirtual    prefix to remove from local user id\n");
 
-	fputs("  -u, --username    specify users's login on server\n", stderr);
-	fputs("  -a, --all         retrieve old and new messages\n", stderr);
-	fputs("  -K, --nokeep      delete new messages after retrieval\n", stderr);
-	fputs("  -k, --keep        save new messages after retrieval\n", stderr);
-	fputs("  -F, --flush       delete old messages from server\n", stderr);
-	fputs("  -n, --norewrite   don't rewrite header addresses\n", stderr);
-	fputs("  -l, --limit       don't fetch messages over given size\n", stderr);
+	P("  -u, --username    specify users's login on server\n");
+	P("  -a, --all         retrieve old and new messages\n");
+	P("  -K, --nokeep      delete new messages after retrieval\n");
+	P("  -k, --keep        save new messages after retrieval\n");
+	P("  -F, --flush       delete old messages from server\n");
+	P("  -n, --norewrite   don't rewrite header addresses\n");
+	P("  -l, --limit       don't fetch messages over given size\n");
+	P("  -w, --warnings   interval between warning mail notification\n");
 
 #if NET_SECURITY
-	fputs("  -T, --netsec      set IP security request\n", stderr);
+	P("  -T, --netsec      set IP security request\n");
 #endif /* NET_SECURITY */
-	fputs("  -S, --smtphost    set SMTP forwarding host\n", stderr);
-	fputs("  -D, --smtpaddress set SMTP delivery domain to use\n", stderr);
-	fputs("  -Z, --antispam,   set antispam response value\n", stderr);
-	fputs("  -b, --batchlimit  set batch limit for SMTP connections\n", stderr);
-	fputs("  -B, --fetchlimit  set fetch limit for server connections\n", stderr);
-	fputs("  -e, --expunge     set max deletions between expunges\n", stderr);
-	fputs("  -r, --folder      specify remote folder name\n", stderr);
+	P("  -S, --smtphost    set SMTP forwarding host\n");
+	P("  -D, --smtpaddress set SMTP delivery domain to use\n");
+	P("  -Z, --antispam,   set antispam response values\n");
+	P("  -b, --batchlimit  set batch limit for SMTP connections\n");
+	P("  -B, --fetchlimit  set fetch limit for server connections\n");
+	P("  -e, --expunge     set max deletions between expunges\n");
+	P("  -r, --folder      specify remote folder name\n");
+#undef P
 	return(-1);
     }
 
