@@ -27,13 +27,17 @@
 extern char *strstr();	/* needed on sysV68 R3V7.1. */
 #endif /* strstr */
 
-static int phase;
+static int pop3_phase;
 #define PHASE_GETAUTH	0
 #define PHASE_GETRANGE	1
 #define PHASE_GETSIZES	2
 #define PHASE_FETCH	3
 #define PHASE_LOGOUT	4
 static int last;
+#ifdef SDPS_ENABLE
+static flag sdps_enable = FALSE;
+char *sdps_envto;
+#endif /* SDPS_ENABLE */
 
 #if OPIE
 static char lastok[POPBUFSIZE+1];
@@ -67,9 +71,9 @@ int pop3_ok (int sock, char *argbuf)
 #endif /* OPIE */
 	    ok = 0;
 	}
-	else if (strcmp(buf,"-ERR") == 0)
+	else if (strncmp(buf,"-ERR", 4) == 0)
 	{
-	    if (phase > PHASE_GETAUTH) 
+	    if (pop3_phase > PHASE_GETAUTH) 
 		ok = PS_PROTOCOL;
 	    /*
 	     * We're checking for "lock busy", "unable to lock", 
@@ -108,7 +112,23 @@ int pop3_getauth(int sock, struct query *ctl, char *greeting)
     char *challenge;
 #endif /* OPIE */
 
-    phase = PHASE_GETAUTH;
+    pop3_phase = PHASE_GETAUTH;
+
+#ifdef SDPS_ENABLE
+    /*
+     * This needs to catch both demon.co.uk and demon.net.
+     * If we see either, and we're in multidrop mode, try to use
+     * the SDPS *ENV extension.
+     */
+    sdps_enable = (MULTIDROP(ctl) && strstr(greeting, "demon."));
+    /*
+     * Use SDPS if configured, regardless of the greeting string
+     * returned from the POP server. (Users accessing demon by a
+     * POP3 proxy may need this)
+     */
+    if (ctl->server.sdps)
+        sdps_enable = ctl->server.sdps;
+#endif /* SDPS_ENABLE */
 
     switch (ctl->server.protocol) {
     case P_POP3:
@@ -119,7 +139,9 @@ int pop3_getauth(int sock, struct query *ctl, char *greeting)
 	  * CompuServe has changed its RPA behavior.  Used to be they didn't
 	  * accept PASS, but I'm told this changed in mid-November 1997.
 	  */
-	 if (strstr(greeting, "csi.com"))
+	 if (strstr(greeting, "csi.com")
+	     && (start = strchr(ctl->remotename, '@'))
+	     && !strcmp("@compuserve.com", start))
 	 {
              /* temporary fix to get back out of cleartext authentication */
              gen_transact(sock, "PASS %s", "dummypass");
@@ -153,7 +175,7 @@ int pop3_getauth(int sock, struct query *ctl, char *greeting)
 	  int i;
 
 	  i = opiegenerator(challenge, !strcmp(ctl->password, "opie") ? "" : ctl->password, response);
-	  if ((i == -2) && (cmd_daemon == -1)) {
+	  if ((i == -2) && !run.poll_interval) {
 	    char secret[OPIE_SECRET_MAX+1];
 	    fprintf(stderr, "Secret pass phrase: ");
 	    if (opiereadpass(secret, sizeof(secret), 0))
@@ -196,12 +218,11 @@ int pop3_getauth(int sock, struct query *ctl, char *greeting)
 	    *++end = '\0';
 
 	/* copy timestamp and password into digestion buffer */
-	msg = (char *)xmalloc((end-start+1) + strlen(ctl->password) + 1);
+	xalloca(msg, char *, (end-start+1) + strlen(ctl->password) + 1);
 	strcpy(msg,start);
 	strcat(msg,ctl->password);
 
 	strcpy(ctl->digest, MD5Digest(msg));
-	free(msg);
 
 	ok = gen_transact(sock, "APOP %s %s", ctl->remotename, ctl->digest);
 	break;
@@ -234,6 +255,9 @@ int pop3_getauth(int sock, struct query *ctl, char *greeting)
      */
     sleep(3); /* to be _really_ safe, probably need sleep(5)! */
 
+    /* we're peek-capable if use of TOP is enabled */
+    peek_capable = !(ctl->fetchall || ctl->keep);
+
     /* we're approved */
     return(PS_SUCCESS);
 }
@@ -359,7 +383,7 @@ static int pop3_getrange(int sock,
     int ok;
     char buf [POPBUFSIZE+1];
 
-    phase = PHASE_GETRANGE;
+    pop3_phase = PHASE_GETRANGE;
 
     /* Ensure that the new list is properly empty */
     ctl->newsaved = (struct idlist *)NULL;
@@ -432,8 +456,7 @@ static int pop3_getrange(int sock,
 			new = save_str(&ctl->newsaved, id, UID_UNSEEN);
 			new->val.status.num = num;
 
-			/* note: ID comparison is caseblind */
-			if (str_in_list(&ctl->oldsaved, id)) {
+			if (str_in_list(&ctl->oldsaved, id, FALSE)) {
 			    new->val.status.mark = UID_SEEN;
 			    str_set_mark(&ctl->oldsaved, id, UID_SEEN);
 			}
@@ -453,7 +476,7 @@ static int pop3_getsizes(int sock, int count, int *sizes)
 {
     int	ok;
 
-    /* phase = PHASE_GETSIZES */
+    /* pop3_phase = PHASE_GETSIZES */
 
     if ((ok = gen_transact(sock, "LIST")) != 0)
 	return(ok);
@@ -481,26 +504,110 @@ static int pop3_is_old(int sock, struct query *ctl, int num)
     if (!ctl->oldsaved)
 	return (num <= last);
     else
-	/* note: ID comparison is caseblind */
         return (str_in_list(&ctl->oldsaved,
-			    str_find (&ctl->newsaved, num)));
+			    str_find(&ctl->newsaved, num), FALSE));
 }
 
-static int pop3_fetch(int sock, struct query *ctl, int number, int *lenp)
-/* request nth message */
+#ifdef UNUSED
+/*
+ * We could use this to fetch headers only as we do for IMAP.  The trouble 
+ * is that there's no way to fetch the body only.  So the following RETR 
+ * would have to re-fetch the header.  Enough messages have longer headers
+ * than bodies to make this a net loss.
+ */
+static int pop_fetch_headers(int sock, struct query *ctl,int number,int *lenp)
+/* request headers of nth message */
 {
     int ok;
-    char buf [POPBUFSIZE+1];
+    char buf[POPBUFSIZE+1];
 
-    /* phase = PHASE_FETCH */
+    /* pop3_phase = PHASE_FETCH */
 
-    gen_send(sock, "RETR %d", number);
+    gen_send(sock, "TOP %d 0", number);
     if ((ok = pop3_ok(sock, buf)) != 0)
 	return(ok);
 
     *lenp = -1;		/* we got sizes from the LIST response */
 
-    return(0);
+    return(PS_SUCCESS);
+}
+#endif /* UNUSED */
+
+static int pop3_fetch(int sock, struct query *ctl, int number, int *lenp)
+/* request nth message */
+{
+    int ok;
+    char buf[POPBUFSIZE+1];
+
+    /* pop3_phase = PHASE_FETCH */
+
+#ifdef SDPS_ENABLE
+    /*
+     * See http://www.demon.net/services/mail/sdps-tech.html
+     * for a description of what we're parsing here.
+     */
+    if (sdps_enable)
+    {
+	int	linecount = 0;
+
+	sdps_envto = (char *)NULL;
+	gen_send(sock, "*ENV %d", number);
+	do {
+	    if (gen_recv(sock, buf, sizeof(buf)))
+            {
+                break;
+            }
+            linecount++;
+            if (linecount == 5)
+            {
+                /* Wrap address with To: <> so nxtaddr() likes it */
+                sdps_envto = malloc(strlen(buf)+7);
+                sprintf(sdps_envto,"To: <%s>",buf);
+            }
+	} while
+	    (buf[0] !='.');
+    }
+#endif /* SDPS_ENABLE */
+
+    /*
+     * Though the POP RFCs don't document this fact, on every POP3 server
+     * I know of messages are marked "seen" only at the time the OK
+     * response to a RETR is issued.
+     *
+     * This means we can use TOP to fetch the message without setting its
+     * seen flag.  This is good!  It means that if the protocol exchange
+     * craps out during the message, it will still be marked `unseen' on
+     * the server.
+     *
+     * However...*don't* do this if we're using keep to suppress deletion!
+     * In that case, marking the seen flag is the only way to prevent the
+     * message from being re-fetched on subsequent runs.
+     *
+     * Also use RETR if fetchall is on.  This gives us a workaround
+     * for servers like usa.net's that bungle TOP.  It's pretty
+     * harmless because fetchall guarantees that any message dropped
+     * by an interrupted RETR will be picked up on the next poll of the
+     * site.
+     *
+     * We take advantage here of the fact that, according to all the
+     * POP RFCs, "if the number of lines requested by the POP3 client
+     * is greater than than the number of lines in the body, then the
+     * POP3 server sends the entire message.").
+     *
+     * The line count passed (99999999) is the maximum value CompuServe will
+     * accept; it's much lower than the natural value 2147483646 (the maximum
+     * twos-complement signed 32-bit integer minus 1)
+     */
+    if (ctl->keep || ctl->fetchall)
+	gen_send(sock, "RETR %d", number);
+    else
+	gen_send(sock, "TOP %d 99999999", number);
+    if ((ok = pop3_ok(sock, buf)) != 0)
+	return(ok);
+
+    *lenp = -1;		/* we got sizes from the LIST response */
+
+    return(PS_SUCCESS);
 }
 
 static int pop3_delete(int sock, struct query *ctl, int number)
@@ -515,7 +622,7 @@ static int pop3_logout(int sock, struct query *ctl)
 {
     int ok;
 
-    /* phase = PHASE_LOGOUT */
+    /* pop3_phase = PHASE_LOGOUT */
 
     ok = gen_transact(sock, "QUIT");
     if (!ok)
